# Copyright © 2021 Arrikto Inc.  All Rights Reserved.

"""Kale SDK.

This script trains an ML pipeline to solve a binary classification task.
"""

import base64
from io import BytesIO

import matplotlib.pyplot as plt
from kale.sdk import artifact, pipeline, step
from sklearn.datasets import make_classification
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split


@step(name="data_loading")
def load(random_state):
    """Create a random dataset for binary classification."""
    rs = int(random_state)
    x, y = make_classification(random_state=rs)
    return x, y


@step(name="data_split")
def split(x, y):
    """Split the data into train and test sets."""
    x, x_test, y, y_test = train_test_split(x, y, test_size=0.1)
    return x, x_test, y, y_test


@artifact(name="plot", path="/home/jovyan/plot.html")
@step(name="plot_data")
def plot(x, y):
    """Create an HTML artifact for KFP UI."""
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    ax.scatter(x[:, 0], y)

    tmpfile = BytesIO()
    fig.savefig(tmpfile, format='png')
    encoded = base64.b64encode(tmpfile.getvalue()).decode('utf-8')

    html = '<img src=\'data:image/png;base64,{}\'>'.format(encoded)
    with open('plot.html', 'w') as f:
        f.write(html)


@step(name="model_training")
def train(x, x_test, y, training_iterations):
    """Train a Logistic Regression model."""
    iters = int(training_iterations)
    model = LogisticRegression(max_iter=iters)
    model.fit(x, y)
    print(model.predict(x_test))


@pipeline(name="binary-classification", experiment="kale-tutorial")
def ml_pipeline(rs=42, iters=100):
    """Run the ML pipeline."""
    x, y = load(rs)
    x, x_test, y, y_test = split(x, y)
    plot(x, y)
    train(x, x_test, y, iters)


if __name__ == "__main__":
    ml_pipeline(rs=42, iters=100)
